package com.link.core.basic.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.link.base.datatask.model.DataTask;
import com.link.base.datatask.service.DataTaskService;
import com.link.base.user.model.CoreUser;
import com.link.core.basic.annotations.JsonParam;
import com.link.core.basic.model.BasicModel;
import com.link.core.basic.query.QueryParams;
import com.link.core.basic.service.BasicService;
import com.link.core.basic.service.BasicServiceException;
import com.link.core.basic.service.KeyGenerateService;
import com.link.core.basic.thread.LinkThreadPoolExecutor;
import com.link.core.cllog.LCLogger;
import com.link.core.modules.export.BigExcelWrite;
import com.link.core.modules.export.ExportFactory;
import com.link.core.modules.export.ExportWriter;
import com.link.core.modules.imports.ImportReader;
import com.link.core.modules.imports.ImportsFactory;
import com.link.core.util.DateUtil;
import com.link.core.util.RedisUtil;
import com.link.core.util.ReflectUtil;
import com.link.core.util.StringUtils;
import com.link.core.util.UserUtil;
import com.link.core.util.WebUtils;
import com.link.core.util.asserts.AssertsUtis;
import com.link.microservice.DataTaskBaseClient;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

@Controller
@RequestMapping({"/link/base"})
public class BasicController<T extends BasicModel> {
   private static final Logger logger = LogManager.getLogger(BasicController.class);
   @Resource
   private DataTaskService dataTaskService;
   @Resource
   private LinkThreadPoolExecutor linkThreadPoolExecutor;
   @Resource
   private DataTaskBaseClient dataTaskBaseClient;
   @Resource
   private KeyGenerateService keyGenerateService;

   @RequestMapping({"/queryAll"})
   @ResponseBody
   public Map<String, Object> queryAll(@JsonParam QueryParams qps, @JsonParam T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(8);

      try {
         qps.preDealSecurity(request);
         T temp = BasicModel.transformClass(entity, qps);
         this.beforQueryAllOrExample(temp, request);
         this.beforQueryAll(temp, request);
         List<T> list = this.getBasicService().queryAll(temp);
         result.put("success", true);
         result.put("rows", list);
         result.put("total", getCount(temp, list.size()));
         result.put("code", "200");
      } catch (BasicServiceException var9) {
         BasicServiceException e = var9;
         result.put("success", false);
         result.put("rows", new ArrayList());
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var10) {
         Exception e = var10;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("rows", new ArrayList());
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/queryByExamplePage"})
   @ResponseBody
   public Map<String, Object> queryByExamplePage(@JsonParam QueryParams qps, @JsonParam T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(8);

      try {
         if (!qps.getPageFlag()) {
            qps.setRows(1000);
            qps.setPage(1);
            qps.setPageFlag(true);
            entity.setRows(1000);
            entity.setPageFlag(true);
            entity.setPage(1);
         }

         qps.preDealSecurity(request);
         T temp = BasicModel.transformClass(entity, qps);
         this.beforQueryAllOrExample(temp, request);
         this.beforQueryExample(temp, request);
         List<T> list = this.getBasicService().queryByExamplePage(temp);
         result.put("success", true);
         result.put("rows", list);
         this.pushCurrentDataCache(list, session.getId(), "base", false);
         result.put("total", getCount(temp, list.size()));
         result.put("code", "200");
      } catch (BasicServiceException var9) {
         BasicServiceException e = var9;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("rows", new ArrayList());
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var10) {
         Exception e = var10;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("rows", new ArrayList());
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/queryFieldsByExamplePage"})
   @ResponseBody
   public Map<String, Object> queryByExamplePage2(@JsonParam QueryParams qps, @JsonParam T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      new HashMap(16);
      String[] stays = qps.getStayFields().split(",");
      qps.setStayFieldsFlag("onlyStayFields");
      Map<String, Object> result = this.queryByExamplePage(qps, entity, session, request, response);
      return result;
   }

   @RequestMapping({"/queryByExamplePage/exportData"})
   @ResponseBody
   public Map<String, Object> exportData(@JsonParam final T entity, @JsonParam final QueryParams qps, HttpSession session, final HttpServletRequest request, HttpServletResponse response) {
      logger.info("导出列表...");
      Map<String, Object> result = new HashMap(4);

      try {
         String asyncExportFlag = qps.getAsyncExportFlag();
         final String exportTaskType = qps.getExportTaskType();
         final String exportCurrentPage = qps.getExportCurrentPage();
         if (StringUtils.isNotBlank(asyncExportFlag) && "Y".equals(asyncExportFlag)) {
            logger.info("异步导出列表...");
            this.linkThreadPoolExecutor.execute(new Runnable() {
               public void run() {
                  DataTask dataTask = null;

                  try {
                     dataTask = BasicController.this.generateDataTask((DataTask)null, "new", exportTaskType, "");
                     BasicController.logger.info("异步导出开始日志标识:" + qps.getExportFileName());
                     String urlPath;
                     if ("Y".equals(exportCurrentPage)) {
                        BasicController.this.beforExportExcel(entity, qps, request);
                        urlPath = BasicController.this.exportDataHandler(entity, qps, request);
                     } else {
                        BasicController.this.beforExportExcelCheck(entity, qps, request);
                        BasicController.this.beforExportExcel(entity, qps, request);
                        urlPath = BasicController.this.streamExportDataHandler(entity, qps, request);
                     }

                     BasicController.this.generateDataTask(dataTask, "success", exportTaskType, urlPath);
                     BasicController.logger.info("异步导出结束日志标识:" + qps.getExportFileName());
                  } catch (Exception var5) {
                     Exception e = var5;
                     LCLogger.withException(e);

                     try {
                        BasicController.this.generateDataTask(dataTask, "error", exportTaskType, e.getMessage());
                        BasicController.logger.info("异步导出异常日志标识:" + qps.getExportFileName());
                     } catch (Exception var4) {
                        Exception e1 = var4;
                        LCLogger.withException("导出任务创建失败：" + e1);
                     }
                  }

               }
            });
         } else {
            logger.info("同步导出列表..." + exportTaskType);
            String urlPath;
            if ("Y".equals(exportCurrentPage)) {
               this.beforExportExcel(entity, qps, request);
               urlPath = this.exportDataHandler(entity, qps, request);
            } else {
               this.beforExportExcelCheck(entity, qps, request);
               this.beforExportExcel(entity, qps, request);
               urlPath = this.streamExportDataHandler(entity, qps, request);
            }

            result.put("url", urlPath);
         }

         result.put("success", true);
         result.put("code", "200");
      } catch (BasicServiceException var11) {
         BasicServiceException e = var11;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var12) {
         Exception e = var12;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   private String exportDataHandler(T entity, QueryParams qps, HttpServletRequest request) throws Exception {
      qps.preDealSecurity(request);
      CoreUser user = UserUtil.getUser();
      if (null != user) {
         user.setExportType(qps.getExportFileType());
      }

      ExportWriter exportWriter;
      if ("xlsx".equals(user.getExportType())) {
         exportWriter = ExportFactory.newExcelInstance(qps);
      } else {
         exportWriter = ExportFactory.newInstance(qps);
      }

      T temp = BasicModel.transformClass(entity, qps);

      while(!qps.getStopExport()) {
         this.beforQueryAllOrExample(temp, request);
         this.beforQueryExample(temp, request);
         List<T> list = this.getBasicService().queryByExamplePage(temp);
         if (qps.getPage() == 1) {
            qps.setTotal(temp.getTotal());
         }

         exportWriter.wirteData(list);
         if ("Y".equals(qps.getExportCurrentPage())) {
            break;
         }

         temp.setPage(qps.getPage());
         temp.setTotalFlag(qps.getTotalFlag());
      }

      return exportWriter.getUrlPath();
   }

   @RequestMapping({"/queryReportPage/exportReportData"})
   @ResponseBody
   public Map<String, Object> exportReportData(@RequestBody final JSONObject jsonObject, HttpSession session, final HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         String asyncExportFlag = jsonObject.getString("asyncExportFlag");
         final String exportTaskType = jsonObject.getString("exportTaskType");
         if (StringUtils.isNotBlank(asyncExportFlag) && "Y".equals(asyncExportFlag)) {
            logger.info("异步导出报表....");
            this.linkThreadPoolExecutor.execute(new Runnable() {
               public void run() {
                  DataTask dataTask = null;

                  try {
                     BasicController.logger.info("异步导出开始日志标识:" + exportTaskType);
                     dataTask = BasicController.this.generateDataTask((DataTask)null, "new", exportTaskType, "");
                     String urlPath = BasicController.this.exportReportDataHandler(jsonObject, request);
                     BasicController.this.generateDataTask(dataTask, "success", exportTaskType, urlPath);
                     BasicController.logger.info("异步导出结束日志标识:" + exportTaskType);
                  } catch (Exception var5) {
                     Exception e = var5;
                     LCLogger.withException(e);

                     try {
                        BasicController.this.generateDataTask(dataTask, "error", exportTaskType, e.getMessage());
                        BasicController.logger.info("异步导出异常日志标识:" + exportTaskType);
                     } catch (Exception var4) {
                        Exception e1 = var4;
                        LCLogger.withException("导出任务创建失败：" + e1);
                     }
                  }

               }
            });
         } else {
            logger.info("同步导出报表...." + exportTaskType);
            String urlPath = this.exportReportDataHandler(jsonObject, request);
            result.put("url", urlPath);
         }

         result.put("success", true);
         result.put("code", "200");
      } catch (BasicServiceException var9) {
         BasicServiceException e = var9;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var10) {
         Exception e = var10;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   private DataTask generateDataTask(DataTask dataTask, String type, String exportTaskType, String urlPath) throws Exception {
      if (dataTask == null) {
         dataTask = new DataTask();
         dataTask.setId(this.keyGenerateService.keyGenerate());
      }

      dataTask.setImportType(exportTaskType);
      dataTask.setType("Export");
      switch (type) {
         case "new":
            dataTask.setTaskStatus("Running");
            dataTask.setTaskMessage("正在导出");
            dataTask.setStartTime(DateUtil.dateToStr(new Date(), "yyyy-MM-dd HH:mm:ss"));
            this.dataTaskBaseClient.insert(JSONObject.parseObject(JSON.toJSONString(dataTask)));
            break;
         case "success":
            dataTask.setTaskStatus("Completed");
            dataTask.setTaskMessage("已完成");
            dataTask.setEndTime(DateUtil.dateToStr(new Date(), "yyyy-MM-dd HH:mm:ss"));
            dataTask.setFeedbackPath(urlPath);
            this.dataTaskBaseClient.update(JSONObject.parseObject(JSON.toJSONString(dataTask)));
            break;
         case "error":
            dataTask.setTaskStatus("Error");
            dataTask.setTaskMessage("导出失败" + urlPath);
            dataTask.setEndTime(DateUtil.dateToStr(new Date(), "yyyy-MM-dd HH:mm:ss"));
            this.dataTaskBaseClient.update(JSONObject.parseObject(JSON.toJSONString(dataTask)));
      }

      return dataTask;
   }

   private String exportReportDataHandler(JSONObject jsonObject, HttpServletRequest request) throws Exception {
      String jsonText = JSONObject.toJSONString(jsonObject);
      QueryParams qps = (QueryParams)JSONObject.parseObject(jsonText, QueryParams.class);
      qps.preDealSecurity(request);
      qps.setRows(jsonObject.getInteger("rows"));
      CoreUser user = UserUtil.getUser();
      if (null != user) {
         user.setExportType(qps.getExportFileType());
      }

      ExportWriter exportWriter;
      if ("xlsx".equals(user.getExportType())) {
         exportWriter = ExportFactory.newExcelInstance(qps);
      } else {
         exportWriter = ExportFactory.newInstance(qps);
      }

      String exportMethodParam = jsonObject.getString("exportMethodParam");
      if (StringUtils.isNull(exportMethodParam)) {
         throw new Exception("必须传入参数类型！");
      } else {
         Object entity = JSONObject.parseObject(jsonText, Class.forName(exportMethodParam));
         Object temp = BasicModel.transformClassByObj(entity, qps);
         String exportMethod = jsonObject.getString("exportMethod");
         if (StringUtils.isNull(exportMethod)) {
            throw new Exception("导出方法不能为空！");
         } else if (!exportMethod.endsWith("Page")) {
            throw new Exception("方法名必须以Page结尾");
         } else {
            Method methodName = ReflectUtil.getMethod(this.getBasicService().getClass(), exportMethod, new Class[]{temp.getClass()});
            if (methodName == null) {
               throw new Exception("传入的方法错误！");
            } else {
               while(!qps.getStopExport()) {
                  List<Object> list = (List)methodName.invoke(this.getBasicService(), temp);
                  if (qps.getPage() == 1) {
                     qps.setTotal((Integer)ReflectUtil.invokeGetter(temp, "total", temp.getClass()));
                  }

                  exportWriter.wirteData(list);
                  if ("Y".equals(qps.getExportCurrentPage())) {
                     break;
                  }

                  ReflectUtil.invokeSetter(temp, "page", qps.getPage(), temp.getClass());
                  ReflectUtil.invokeSetter(temp, "totalFlag", qps.getTotalFlag(), temp.getClass());
               }

               return exportWriter.getUrlPath();
            }
         }
      }
   }

   @RequestMapping({"/importData"})
   @ResponseBody
   public Map<String, Object> importData(T entity, QueryParams qps, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      logger.info("导入列表。。。");
      Map<String, Object> result = new HashMap(4);

      try {
         ImportReader importReader = ImportsFactory.newInstance(qps, request);
         List<T> list = importReader.readList(entity.getClass());
         this.importCsvData(list, qps, request);
         result.put("success", true);
         result.put("total", list.size());
         result.put("code", "200");
      } catch (BasicServiceException var9) {
         BasicServiceException e = var9;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var10) {
         Exception e = var10;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @Transactional(
      rollbackFor = {BasicControllerException.class}
   )
   protected void importCsvData(List<T> list, QueryParams qps, HttpServletRequest request) throws Exception {
      this.beforeImport(list, qps, request);

      for(int i = 0; list != null && i < list.size(); ++i) {
         T record = (BasicModel)list.get(i);
         this.beforeImport(record, request);
         this.getBasicService().insert(record);
      }

   }

   protected void beforeImport(List<T> list, QueryParams qps, HttpServletRequest request) throws Exception {
   }

   public void beforeImport(T entity, HttpServletRequest request) throws Exception {
   }

   @RequestMapping({"/deepCopy"})
   @ResponseBody
   public Map<String, Object> deepCopy(T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         throw new BasicServiceException("深度复制失败，请检查程序代码。");
      } catch (BasicServiceException var7) {
         BasicServiceException e = var7;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", "200");
      } catch (Exception var8) {
         Exception e = var8;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/insert"})
   @ResponseBody
   public Map<String, Object> insert(@JsonParam T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         this.beforInsert(entity, request);
         this.beforUpsert(entity, request);
         this.getBasicService().insert(entity);
         T newEntry = this.getBasicService().queryById(entity);
         result.put("newRow", newEntry);
         result.put("success", true);
         result.put("code", "200");
      } catch (BasicServiceException var7) {
         BasicServiceException e = var7;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var8) {
         Exception e = var8;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping(
      value = {"/preDefaultValue"},
      method = {RequestMethod.POST},
      produces = {"application/json"}
   )
   @ResponseBody
   public Map<String, Object> preDefaultValue(T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         result.put("success", true);
         result.put("result", this.getBasicService().preDefaultValue(entity));
         result.put("code", "200");
      } catch (BasicServiceException var7) {
         BasicServiceException e = var7;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var8) {
         Exception e = var8;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/batchInsert"})
   @ResponseBody
   public Map<String, Object> batchInsert(@RequestBody List<T> list, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      Exception e;
      try {
         e = null;

         for(int i = 0; list != null && i < list.size(); ++i) {
            T entity = (BasicModel)list.get(i);
            this.beforInsert(entity, request);
            this.beforUpsert(entity, request);
         }

         this.getBasicService().batchInsert(list);
         result.put("success", true);
         result.put("result", list);
         result.put("code", "200");
      } catch (BasicServiceException var8) {
         logger.error("BasicController捕获异常: ", var8);
         result.put("success", false);
         result.put("result", var8.getMessage());
         result.put("detailMessage", var8.getDetailMessage());
         result.put("code", var8.getCode());
      } catch (Exception var9) {
         e = var9;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/newBatchInsert"})
   @ResponseBody
   public Map<String, Object> newBatchInsert(@RequestBody List<T> list, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         this.getBasicService().batchInsert(list, true);
         result.put("success", true);
         result.put("result", list);
         result.put("code", "200");
      } catch (BasicServiceException var7) {
         BasicServiceException e = var7;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var8) {
         Exception e = var8;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/batchUpdate"})
   @ResponseBody
   public Map<String, Object> batchUpdate(@RequestBody List<T> list, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      Exception e;
      try {
         e = null;

         for(int i = 0; list != null && i < list.size(); ++i) {
            T entity = (BasicModel)list.get(i);
            this.beforUpdate(entity, request);
            this.beforUpsert(entity, request);
         }

         this.getBasicService().batchUpdate(list);
         result.put("success", true);
         result.put("result", list);
         result.put("code", "200");
      } catch (BasicServiceException var8) {
         logger.error("BasicController捕获异常: ", var8);
         result.put("success", false);
         result.put("result", var8.getMessage());
         result.put("detailMessage", var8.getDetailMessage());
         result.put("code", var8.getCode());
      } catch (Exception var9) {
         e = var9;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/batchUpsert"})
   @ResponseBody
   public Map<String, Object> batchUpsert(@RequestBody List<T> list, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      Exception e;
      try {
         e = null;

         for(int i = 0; list != null && i < list.size(); ++i) {
            T entity = (BasicModel)list.get(i);
            this.beforUpsert(entity, request);
         }

         this.getBasicService().batchUpsert(list);
         result.put("success", true);
         result.put("result", list);
         result.put("code", "200");
      } catch (BasicServiceException var8) {
         logger.error("BasicController捕获异常: ", var8);
         result.put("success", false);
         result.put("result", var8.getMessage());
         result.put("detailMessage", var8.getDetailMessage());
         result.put("code", var8.getCode());
      } catch (Exception var9) {
         e = var9;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/queryById"})
   @ResponseBody
   public Map<String, Object> queryById(@JsonParam T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         T record = this.getBasicService().queryById(entity);
         List<T> list = new ArrayList();
         list.add(record);
         this.pushCurrentDataCache(list, session.getId(), "base", false);
         result.put("result", record);
         result.put("success", true);
         result.put("code", "200");
      } catch (BasicServiceException var8) {
         BasicServiceException e = var8;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var9) {
         Exception e = var9;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/upsert"})
   @ResponseBody
   public Map<String, Object> upsert(@JsonParam T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         this.beforUpsert(entity, request);
         this.checkData(entity, request);
         if (!this.getBasicService().isInsertFlag(entity) && !this.checkExistDataCache("base", session, entity)) {
            throw new BasicServiceException("非法更新数据！");
         }

         this.getBasicService().upsert(entity);
         T newEntry = this.getBasicService().queryById(entity);
         result.put("newRow", newEntry);
         result.put("success", true);
         result.put("code", "200");
      } catch (BasicServiceException var7) {
         BasicServiceException e = var7;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var8) {
         Exception e = var8;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/update"})
   @ResponseBody
   public Map<String, Object> update(@JsonParam T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         this.beforUpdate(entity, request);
         this.beforUpsert(entity, request);
         this.checkData(entity, request);
         if (!this.checkExistDataCache("base", session, entity)) {
            throw new BasicServiceException("非法更新数据！");
         }

         this.getBasicService().update(entity);
         T newEntry = this.getBasicService().queryById(entity);
         result.put("newRow", newEntry);
         result.put("success", true);
         result.put("code", "200");
      } catch (BasicServiceException var7) {
         BasicServiceException e = var7;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var8) {
         Exception e = var8;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/deleteById"})
   @ResponseBody
   public Map<String, Object> deleteById(@JsonParam T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      logger.info("删除一条列表信息...");
      Map<String, Object> result = new HashMap(4);

      try {
         this.beforDelete(entity, request);
         this.checkData(entity, request);
         if (!this.checkExistDataCache("base", session, entity)) {
            throw new BasicServiceException("非法删除数据！");
         }

         this.getBasicService().deleteById(entity);
         result.put("success", true);
         result.put("code", "200");
      } catch (BasicServiceException var7) {
         BasicServiceException e = var7;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var8) {
         Exception e = var8;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   public BasicService<T> getBasicService() throws Exception {
      throw new BasicControllerException("获取Basic Service 失败，请确认程序代码是否正确！");
   }

   public T postDefaultValue(T entity) throws Exception {
      return entity;
   }

   public void beforQueryAllOrExample(T entity, HttpServletRequest request) throws Exception {
   }

   public void beforQueryAll(T entity, HttpServletRequest request) throws Exception {
   }

   public void beforQueryExample(T entity, HttpServletRequest request) throws Exception {
   }

   public void beforUpdate(T entity, HttpServletRequest request) throws Exception {
   }

   public void beforInsert(T entity, HttpServletRequest request) throws Exception {
   }

   public void beforDelete(T entity, HttpServletRequest request) throws Exception {
   }

   public void beforUpsert(T entity, HttpServletRequest request) throws Exception {
   }

   public void checkData(T entity, HttpServletRequest request) throws Exception {
   }

   public void beforReadExcel(T entity, QueryParams qps, HttpServletRequest request) throws Exception {
   }

   public void beforeImportExcel(T entity, HttpServletRequest request) throws Exception {
   }

   public void importCheckExcel(List<T> entity, HttpServletRequest request) throws Exception {
   }

   public void importHeadAndItemExcel(List<T> entity, HttpServletRequest request) throws Exception {
   }

   public void beforExportExcel(T entity, QueryParams qps, HttpServletRequest request) throws Exception {
   }

   public void beforExportExcelCheck(T entity, QueryParams queryParams, HttpServletRequest request) throws Exception {
      T t = (BasicModel)WebUtils.deepCopy(entity);
      QueryParams qps = (QueryParams)WebUtils.deepCopy(queryParams);
      qps.setPageFlag(true);
      T temp = BasicModel.transformClass(t, qps);
      temp.setOnlyCountFlag(true);
      this.beforQueryAllOrExample(temp, request);
      this.beforQueryExample(temp, request);
      List<T> list = this.getBasicService().queryByExamplePage(temp);
      if (!CollectionUtils.isEmpty(list)) {
         int total = ((BasicModel)list.get(0)).getTotal();
         String data = ((BasicModel)list.get(0)).getClass().getSimpleName();
         logger.info("当前导出对象-" + data);
         String exportData = RedisUtil.getProperty("exportEntityLimit", "0");
         if (StringUtils.isBlank(exportData)) {
            throw new BasicServiceException("企业参数配置参数为空exportEntityLimit");
         } else if (!exportData.contains(data) && total > 1000000) {
            throw new BasicServiceException("导出数据超出最大导出数据量1000000");
         } else {
            String exportLImitNumStr = RedisUtil.getCropProperty("EXPORT_LIMIT_NUM", "0");
            int exportLImitNum = StringUtils.isBlank(exportLImitNumStr) ? 1000000 : Integer.parseInt(exportLImitNumStr);
            DecimalFormat df = new DecimalFormat("#.##");
            BigDecimal sourceBd = (new BigDecimal(total)).divide(new BigDecimal(10000)).setScale(2, 4);
            BigDecimal errorBd = (new BigDecimal(exportLImitNum)).divide(new BigDecimal(10000)).setScale(2, 4);
            AssertsUtis.isTrue(total > exportLImitNum, ":待导出数据量" + df.format(sourceBd) + "万超过" + df.format(errorBd) + "万限制，请增加筛选条件");
         }
      }
   }

   public void pushCurrentDataCache(List<T> list, String sessionId, String key, boolean flag) {
   }

   public static boolean checkExistDataCache(String key, String sessionId, String id) {
      return true;
   }

   public boolean checkExistDataCache(String key, HttpSession session, T entity) {
      BasicModel basicModel = null;
      if (entity instanceof BasicModel) {
         basicModel = entity;
      }

      if (basicModel == null) {
         throw new BasicServiceException("参数格式错误，请检查程序！");
      } else {
         String sessionId = session.getId();
         String basicModelId = basicModel.getId();
         return checkExistDataCache(key, sessionId, basicModelId);
      }
   }

   public static Integer getCount(BasicModel basicModel, int len) {
      int total = basicModel.getTotal();
      return basicModel.getPageFlag() && !basicModel.getTotalFlag() ? basicModel.getStartRow() + total - 1 : total;
   }

   public static void responseJson(HttpServletRequest request, HttpServletResponse response, String json) {
      response.setCharacterEncoding("UTF-8");
      response.setContentType("application/json; charset=utf-8");
      PrintWriter out = null;

      try {
         out = response.getWriter();
         out.append(json);
         out.flush();
      } catch (IOException var8) {
         IOException e = var8;
         logger.error("BasicController捕获异常: ", e);
      } finally {
         if (out != null) {
            out.close();
         }

      }

   }

   @RequestMapping({"/queryRefactorReport"})
   @ResponseBody
   public Map<String, Object> queryRefactorReport(@JsonParam QueryParams qps, @JsonParam T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         qps.preDealSecurity(request);
         T temp = BasicModel.transformClass(entity, qps);
         this.beforQueryAllOrExample(temp, request);
         this.beforQueryExample(temp, request);
         this.getBasicService().queryByExamplePage(temp);
         List<Map<String, String>> refactorResult = temp.getReportResult();
         result.put("success", true);
         result.put("result", refactorResult);
         result.put("code", "200");
      } catch (BasicServiceException var9) {
         BasicServiceException e = var9;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var10) {
         Exception e = var10;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping(
      value = {"/queryByExamplePage/streamExport"},
      method = {RequestMethod.POST}
   )
   @ResponseBody
   public Map<String, Object> streamExport(@JsonParam final T entity, @JsonParam final QueryParams qps, HttpSession session, final HttpServletRequest request, HttpServletResponse response) {
      logger.info("导出列表...");
      Map<String, Object> result = new HashMap(4);

      try {
         String asyncExportFlag = qps.getAsyncExportFlag();
         final String exportTaskType = qps.getExportTaskType();
         final String exportCurrentPage = qps.getExportCurrentPage();
         if (StringUtils.isNotBlank(asyncExportFlag) && "Y".equals(asyncExportFlag)) {
            logger.info("异步导出列表...");
            this.linkThreadPoolExecutor.execute(new Runnable() {
               public void run() {
                  DataTask dataTask = null;

                  try {
                     dataTask = BasicController.this.generateDataTask((DataTask)null, "new", exportTaskType, "");
                     String urlPath;
                     if ("Y".equals(exportCurrentPage)) {
                        urlPath = BasicController.this.exportDataHandler(entity, qps, request);
                     } else {
                        urlPath = BasicController.this.streamExportDataHandler(entity, qps, request);
                     }

                     BasicController.this.generateDataTask(dataTask, "success", exportTaskType, urlPath);
                  } catch (Exception var5) {
                     Exception e = var5;
                     LCLogger.withException(e);

                     try {
                        BasicController.this.generateDataTask(dataTask, "error", exportTaskType, e.getMessage());
                     } catch (Exception var4) {
                        Exception e1 = var4;
                        LCLogger.withException("导出任务创建失败：" + e1);
                     }
                  }

               }
            });
         } else {
            logger.info("同步导出列表...");
            String urlPath;
            if ("Y".equals(exportCurrentPage)) {
               urlPath = this.exportDataHandler(entity, qps, request);
            } else {
               urlPath = this.streamExportDataHandler(entity, qps, request);
            }

            result.put("url", urlPath);
         }

         result.put("success", true);
         result.put("code", "200");
      } catch (BasicServiceException var11) {
         BasicServiceException e = var11;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var12) {
         Exception e = var12;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   private String streamExportDataHandler(T entity, QueryParams qps, HttpServletRequest request) throws Exception {
      qps.preDealSecurity(request);
      CoreUser user = UserUtil.getUser();
      if (null != user) {
         user.setExportType(qps.getExportFileType());
      }

      ExportWriter exportWriter;
      if ("xlsx".equals(user.getExportType())) {
         exportWriter = ExportFactory.newExcelInstance(qps);
      } else {
         exportWriter = ExportFactory.newInstance(qps);
      }

      T temp = BasicModel.transformClass(entity, qps);

      while(!qps.getStopExport()) {
         this.beforQueryAllOrExample(temp, request);
         this.beforQueryExample(temp, request);
         if (qps.getPage() == 1) {
            qps.setTotal(temp.getTotal());
         }

         exportWriter.writeData(entity, this.getBasicService());
         if ("Y".equals(qps.getExportCurrentPage())) {
            break;
         }

         temp.setPage(qps.getPage());
         temp.setTotalFlag(qps.getTotalFlag());
      }

      return exportWriter.getUrlPath();
   }

   @RequestMapping({"/queryByExamplePage/exportBigData"})
   @ResponseBody
   public Map<String, Object> exportBigData(@JsonParam T entity, @JsonParam QueryParams qps, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      logger.info("导出列表...");
      Map<String, Object> result = new HashMap(4);

      try {
         String asyncExportFlag = qps.getAsyncExportFlag();
         String exportTaskType = qps.getExportTaskType();
         String exportCurrentPage = qps.getExportCurrentPage();
         String urlPath;
         if (StringUtils.isNotBlank(asyncExportFlag) && "Y".equals(asyncExportFlag)) {
            logger.info("异步导出列表...");
            urlPath = this.exportBigData(entity, qps, request);
            result.put("url", urlPath);
         } else {
            logger.info("同步导出列表...");
            urlPath = this.exportBigData(entity, qps, request);
            result.put("url", urlPath);
         }

         result.put("success", true);
         result.put("code", "200");
      } catch (BasicServiceException var11) {
         BasicServiceException e = var11;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var12) {
         Exception e = var12;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   public String exportBigData(T entity, QueryParams qps, HttpServletRequest request) throws Exception {
      qps.preDealSecurity(request);
      CoreUser user = UserUtil.getUser();
      if (null != user) {
         user.setExportType(qps.getExportFileType());
      }

      BigExcelWrite bigExcelWriter = new BigExcelWrite(qps, qps.getExportFileType());
      T temp = BasicModel.transformClass(entity, qps);
      new ArrayList();

      while(!qps.getStopExport()) {
         this.beforQueryAllOrExample(temp, request);
         this.beforQueryExample(temp, request);
         List<T> list = this.getBasicService().queryByExamplePage(temp);
         if (qps.getPage() == 1) {
            qps.setTotal(temp.getTotal());
         }

         bigExcelWriter.wirteData(list);
         if ("Y".equals(qps.getExportCurrentPage())) {
            break;
         }

         temp.setPage(qps.getPage());
         temp.setTotalFlag(qps.getTotalFlag());
      }

      return bigExcelWriter.getUrlPath();
   }
}
